<?php
// $Id: 

/**
 *
 * @author rschaer
 *
 */


class FGImage {
  private $path, $name, $folder, $parent, $fid, $isDir;

  /**
   * Constructor
   * @param $path
   *  The path to the image
   * @param $options
   *  an assoziative array containing some options. The following options are possible:
   *    - dir: can be true or false. If set to true the image behaves as a dir! Default is set to false
   */
  public function __construct($path, $options = array()) {
    $this->setPath($path);

    $fid = db_query("SELECT fid FROM {file} WHERE uri = :uri", array(':uri' => $path))->fetchField();
    $this->setFid($fid);
    
    if (isset($options['dir'])) {
      $this->setIsDir($options['dir']);
    }
    else {
      $this->setIsDir(FALSE);
    }
    
    // Split up the Image Path, so we can work on it
    $arPath = explode('/', $path);
    
    $this->setFolder($arPath);
    $this->setParent($arPath);
    $this->setName($arPath);
  }

  /**
   * Render the html for displaying the image
   * @return string html
   */
  public function renderHtml() {
    $cache = fast_gallery_get_cache();
    $path = $this->getPath(); 
    if ($this->isDir) { //incase a dir we return a special image
      $path = drupal_get_path('module', 'fast_gallery') . '/images/doc.screenshot.jpg';
    }
    $cache->createthumb($path, 150, 100);
    return theme('image', array('path' => $path . '.thumb'));
  }
  /**
   * Given a path and index n, removes n number of elements from the end of
   * the path (delimited by '/').
   *
   * @param path_ar
   *   Path.
   * @param n
   *   Number of elements to remove from end of the path.
   */
  private function substractFromPath($path_ar, $n) {
    $path_ar = array_slice($path_ar, 0, - $n);
    return implode('/', $path_ar);
  }
  
  
  // All the accessor mehtods
  // -------------------------

  public function setIsDir($isDir) {
    if ($isDir) {
      $this->isDir = TRUE;
    }
    else {
      $this->isDir = FALSE;
    }
  }

  public function isDir() {
    return $this->isDir;
  }

  public function getFid() {
    return $this->fid;
  }

  public function setFid($fid) {
    $this->fid = $fid;
  }

  public function getFolder() {
    return $this->folder;
  }

  public function setFolder($arPath) {
    if (!$this->isDir()) {
      $this->folder = $this->substractFromPath($arPath, 1);
    }
    else {
      $this->folder = implode("/", $arPath);
    }
  }

  public function getParent() {
    return $this->parent;
  }

  public function setParent($arPath) {
    $this->parent = $this->substractFromPath($arPath, 2);
  }
  /**
   * Getter method
   * @return string $path
   */
  public function getPath() {
    return $this->path;
  }

  /**
   * Setter method
   * @param $path string
   */
  public function setPath($path) {
    $this->path = $path;
  }

  /**
   * Getter method
   * @return string $path
   */
  public function getName() {
    return $this->name;
  }

  /**
   * Setter method
   * @param $name string
   */
  public function setName($arPath) {
    $this->name = $arPath[count($arPath)-1];
  }

}