<?php

/**
 * Callback function for admin settings
 */
function fast_gallery_general_settings_form() {

  $form['fg_setup'] = array(
    '#type' => 'fieldset',
    '#title' => t('Setup'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['fg_setup']['fast_gallery_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Path to Gallery'),
    '#default_value' => variable_get('fast_gallery_path', 'sites/default/files'),
    '#description' => t("Path to images from which albums should be created. Relative to site's root."),
  );

  $form['fg_setup']['fast_gallery_title'] = array(
    '#type' => 'textfield',
    '#title' => t('Gallery Title'),
    '#default_value' => variable_get('fast_gallery_title', 'Fast Gallery'),
    '#description' => t('Title to display at the top of the gallery.'),
    '#size' => 30,
  );

  //TODO: Need to process the different engines
  $engines = glob(drupal_get_path('module', 'fast_gallery') . '/storageengine/' . '*storage.inc', GLOB_MARK);
  
  $form['fg_setup']['fg_storage_engine'] = array(
    '#type' => 'select',
    '#title' => t('Storage engine'),
    '#default_value' => variable_get('fg_storage_engine',FG_DEFAULT_STORAGE_ENGINE),
    '#options' => array(FG_DEFAULT_STORAGE_ENGINE => 'Default'),
  );

  $js_framework_options = array();
  $js_framework_options[0] = t('<none>');
  $js_framework_default = 0;
  if (module_exists('thickbox'))
  $js_framework_options[1] = t('Thickbox');
  if (module_exists('lightbox2')) {
    $js_framework_default = 2;
    $js_framework_options[2] = t('Lightbox2 (no slideshow)');
    $js_framework_options[3] = t('Lightbox2 (with slideshow)');
  }

  $form['fg_setup']['fast_gallery_js_framework'] = array(
    '#type' => 'select',
    '#title' => t('Choose your Javascript framework'),
    '#options' => $js_framework_options,
    '#default_value' => variable_get('fast_gallery_js_framework', $js_framework_default),
    '#description' => t('Thickbox and Lightbox are the two options for displaying images.'),
  );

  $ic_link = l('Imagecache', 'admin/build/imagecache');
  $form['fast_gallery_operations'] = array(
    '#type' => 'fieldset',
    '#title' => t('Operations'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#tree' => FALSE,
    '#description' => t('Remember to first click Save Settings (if you made any changes) before performing the below operations. Clear the database whenever you change the gallery path. Rescan the database whenever you add images, delete images, move images, or restructure the hierarchy.  Flush Internal Cache to delete and force recaching of Internal Cache thumbnails. If you are using Imagecache, you need to flush thumbnails on the Imagecache settings page: ') . $ic_link,
  );

  $form['fast_gallery_operations']['fast_gallery_clear'] = array(
    '#type' => 'button',
    '#value' => t('Clear DB'),
    '#ajax' => array(
      'callback' => 'fast_gallery_ajax_handler',
      'method' => 'replace',
      'effect' => 'fade',
      'progress' => array(
        'type' => 'bar',
        'message' => t('Clearing DB')
  ),
      'wrapper' => 'attach-wrapper',
  )
  );

  $form['fast_gallery_operations']['fast_gallery_rescan'] = array(
    '#type' => 'button',
    '#value' => t('Rescan'),
    '#ajax' => array(
      'callback' => 'fast_gallery_ajax_handler',
      'method' => 'replace',
      'effect' => 'fade',
      'progress' => array(
        'type' => 'bar',
        'message' => t('Scan for new pictures...')
      ),
      'wrapper' => 'attach-wrapper',
    )
  );

  $form['fast_gallery_operations']['fast_gallery_flush_thumbs'] = array(
    '#type' => 'button',
    '#value' => t('Flush Internal Cache'),
    '#ahah' => array(
      'path' => 'fast_gallery/ajax/flush',
      'method' => 'replace',
      'effect' => 'fade',
      'progress' => array(
        'type' => 'bar',
        'message' => t('Flushing thumbnails')
      ),
      'wrapper' => 'attach-wrapper',
    )
  );

  return system_settings_form($form);
}

/**
 * validation function for the settinsg form
 * @param $form
 * @param $form_state
 */
function fast_gallery_general_settings_form_validate($form, &$form_state) {
  $path = $form_state['values']['fast_gallery_path'];
  if (!is_dir($path)) {
    form_error($form['fg_setup']['fast_gallery_path'], t('This path does not exist.'));
  }
  $last_token = substr($path, -1);
  if ($last_token != '/') {
    $path .= '/';
    $form_state['values']['fast_gallery_path'] = $path;
  }
}


function fast_gallery_ajax_handler($form, $form_state) {
  $fg = fast_gallery_get_FastGallery();
  switch ($form_state['clicked_button']['#id']) {
    case 'fast-gallery-flush-thumbs': //clear cache
      $cache = fast_gallery_get_cache();
      //$cache->flushThumbs()
      break;
    case 'edit-fast-gallery-rescan': //reindex images
      $fg->rescanGallery();
      break;
      
    case 'edit-fast-gallery-clear': //clear db
      $fg->clearDb();
      break;
  }  
}