<?php

/**
 * @file
 * This is a helper class doing most of the database calls.
 *
 * @author Raphael Sch�r - www.schaerwebdesign.ch
 */

include_once('FGImage.class.php');

class FastGallery {
  private static $instance = null;

  /**
   * We are implementing a singleton pattern
   */
  private function __construct() {
  }

  public function getInstance() {
    if (is_null(self :: $instance)) {
      self :: $instance = new self;
    }
    return self :: $instance;
  }

  /**
   * Checks for gallery additions/deletions only. Leaves existing files alone.
   */
  public function rescanGallery() {
    $path = variable_get('fast_gallery_path', 'sites/default/files/');
    $path = utf8_decode($path);

    if (!is_dir($path)) {
      watchdog('fast_gallery', 'No gallery path specified! Do it at !link', array('!link' => l('admin/config/fast_gallery', 'admin/config/fast_gallery')));
    }
    // Get current file list hierarchy
    $files = $this->exploreDir($path, TRUE);

    // Delete non-existent files
    //$this->deletePics();

    // Process the files
    $FgImgContainer = array();
    $this->processFiles($files, $FgImgContainer);
    
    $storage = $this->getStorageEngine();
    $storage->storeImages($FgImgContainer);
    
    // Check for orphan images/folders and fix it
    //$this->checkHierarchy();
  }
  
  /**
   * Return the storage engine
   * @return Istorage
   */
  public function getStorageEngine() {
    $file = variable_get('fg_storage_engine',FG_DEFAULT_STORAGE_ENGINE);
    include_once(drupal_get_path('module', 'fast_gallery') . '/storageengine/' . $file);
    switch ($file) {
      case 'default.storage.inc':
        return new DefaultStorage();
    }
  }
  
/**
   * Process the images (which are right now only a path) and
   * make nice FGImages out of them so that they can be
   * passed to the storage engine
   *
   * @param ar_files
   *   Array of files, as returned by exploreDir()
   * @param fGImagesContainer
   *   Array that collects the created FGImages objects
   */
  private function processFiles($ar_files, &$fGImagesContainer) {
    foreach ($ar_files as $key => $value) {
      // If we found a folder, recurse through it
      if (is_array($value)) {
        $this->processFiles($value, $fGImagesContainer);
      } 
      else {
        // Handle files with special characters.
        $value_slash = str_replace('\\', '/', utf8_encode($value));
        $fGImagesContainer[] = new FGImage($value_slash);
      }

      //we don't care about empty arrays
      if (is_array($value) && count($value) < 1) {
        continue;
      }
    }
    //return $fgImagesContainer;
  }
  
  /**
   * clearing the db -> removing all entries from the db
   * 
   */
  public function clearDb() {
    $storage = $this->getStorageEngine();
    $storage->clearDb();
  }


  /**
   * Get all images in given directory.
   *
   * @param path
   *   Absolute path of the directory
   * @param recursive
   *   Specify whether to recurse through subdirectories.
   * @return
   *   Array of arrays of image files. Each array element corresponds to
   *   an image type. If recursive is on, returns an array tree hierarchy.
   */
  private function exploreDir($path = '', $recursive = FALSE) {
    // List of image extensions
    $exts = array (
      'jpg',
    'jpeg',
     'png',
     'gif',
     'bmp',
     /*'flv',
     'mov',
     'wmv',
     'asx',
     'swf',
     'pdf',*/
    );
    // Get all image files of each file type
    $files = array ();
    foreach ($exts as $ext) {
      $pattern = sql_regcase($path . '*.' . $ext);
      $files[] = glob($pattern);
    }

    // Recurse through subdirectories if necessary
    if ($recursive) {
      $dirs = glob($path . '*', GLOB_MARK | GLOB_ONLYDIR);
      if (is_array($dirs)) {
        foreach ($dirs as $dir) {
          $files[] = $this->exploreDir($dir, TRUE);
        }
      }
    }
    return $files;
  }
  
  /**
   * Building the array for the breadcrumbs
   * @return array
   */
  public function buildBreadCrumbs() {
    $bc = array();
    $path = '';
    foreach (arg() as $item) {
      $path .= $item . '/';
      $bc[] = l($item, $path);
    }
    return $bc;
  }
}