<?php
include_once('Istorage.php');

class DefaultStorage implements Istorage {
  /**
   * Return all images that are stored on a given path
   *
   * @param $path String
   * @return array - an array of FGImages
   */
  public function getImages($path) {
    $base_path = variable_get('fast_gallery_path','sites/default/files');
    $path = $base_path . $path;
    
    //incase we have a / at the end -> remove it.
    $arTmp = explode("/", $path);
    if ($arTmp[sizeof($arTmp)-1] == '') {
      array_pop($arTmp);
    }
    $path = implode("/", $arTmp);
    
    //get the images
    $row = db_query("SELECT uri, fgh.foid FROM {fast_gallery} as fg, {file} as f, {fast_gallery_hierarchy} as fgh 
            WHERE fg.foid = fgh.foid 
            AND fgh.folder = :path 
            AND fg.fid = f.fid", array(':path' => $path), array('fetch' => PDO::FETCH_ASSOC));
    
    $return = array();
    $foid = 0;
    foreach ($row as $item) {
      $return[] = new FGImage($item['uri']);
      $foid = $item['foid'];
    }
    
    $row = db_query("SELECT folder FROM {fast_gallery_hierarchy} WHERE pid = :foid", array(':foid' => $foid), array('fetch' => PDO::FETCH_ASSOC));
    foreach ($row as $folder) {
      $return[] = new FGImage($folder['folder'], array('dir' => TRUE));
    }
    return $return;
  }

  /**
   * implementing the interface. Passing an array of
   * FGImages and save them into the db
   * @param array $arImages
   *  an array of FGImages
   *
   */
  public function storeImages($arImages) {
    global $user;
    
    //TODO: This makes the APP slow -> Rebuilding the whole
    // Gallery every time!
    db_query("TRUNCATE TABLE  `fast_gallery_hierarchy`");
    db_query("TRUNCATE TABLE  `fast_gallery`");
    
    foreach ($arImages as $image) {
      $file = new stdClass();
      $file->uri = $image->getPath();
      $file->uid = $user->uid;
      $file->filename = $image->getName();
      $file->fid = $image->getFid();
      file_save($file);
      $file->foid = $this->storeHierarchy($image);
      
      /*$args = array(':fid' => $file->fid, ':foid' => $file->foid);
      $count = db_query("SELECT COUNT(fid) as count FROM {fast_gallery} 
                          WHERE fid=:fid AND foid=:foid", $args)->fetchField();
      if ($count == 0) {
        
      }*/
      drupal_write_record('fast_gallery', $file);
      
    }
  }
  
  /**
   * (non-PHPdoc)
   * @see sites/all/modules/fast_gallery/storageengine/Istorage#clearDb()
   */
  public function clearDb() {
    $row = db_query("SELECT fid FROM {fast_gallery}", array(), array('fetch' => PDO::FETCH_ASSOC));
    foreach ($row as $item) {
      db_query("DELETE FROM {file} WHERE fid=:fid", array(':fid' => $item['fid']));
    } 
    db_query("TRUNCATE TABLE {fast_gallery_hierarchy}");
    db_query("TRUNCATE TABLE {fast_gallery}");
  }
  
  /**
   * In order to retrieve the hierarchy we need to store this information
   * in a special table
   * @param FGImage $image
   * @return unknown_type
   */
  private function storeHierarchy($image) {
    static $folders = array();

    //a little cache to reduce load on db -> we only have to run this for one image per folder
    if (!in_array($image->getFolder(), $folders)) {
      $folder = db_query("SELECT folder AS c FROM {fast_gallery_hierarchy}
              WHERE folder = :folder", array(':folder' => $image->getFolder()))->fetchField();
      
      if (!$folder) { //incase we don't have this folder in our db
        $dbObject['folder'] = $image->getFolder();
        $dbObject['pid'] = db_query("SELECT foid FROM {fast_gallery_hierarchy} 
                            WHERE folder = :folder", array(':folder' => $image->getParent()))->fetchField();
        drupal_write_record('fast_gallery_hierarchy', $dbObject);
        
        //TODO: needs to be checked!!! needs to return the foid, so that we can make
        //the correct entry into fast_gallery
        $foid = db_query("SELECT MAX(foid) FROM {fast_gallery_hierarchy}")->fetchField();
        $folders[$foid] = $image->getFolder();
        return $foid;
      }
    }
    $ar = array_flip($folders);
    return $ar[$image->getFolder()];
  }
  
}